//! Label manipulation query functions

use core::fmt;

use crate::query::ops::Operable;

use super::QryFunc;

/// MetricsQL's label_map query function
pub fn mql_label_map<'a>(
    qry_expr: impl Operable + 'a,
    label: &'a str,
    src_dst_pairs: &'a [&'a str],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + 'a> {
    QryFunc::new(move |f| {
        write!(
            f,
            "label_map({qry_expr},\"{label}\",\"{pairs}\")",
            pairs = src_dst_pairs.join("\",\"")
        )
    })
}

fn basic_label_fn<'a>(
    fn_name: &'static str,
    qry_expr: impl Operable + 'a,
    labels: &'a [&'a str],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + 'a> {
    QryFunc::new(move |f| {
        write!(
            f,
            "{fn_name}({qry_expr},\"{labels}\")",
            labels = labels.join("\",\"")
        )
    })
}

/// MetricsQL's sort_by_label query function
pub fn mql_sort_by_label<'a>(
    qry_expr: impl Operable + 'a,
    labels: &'a [&'a str],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + 'a> {
    basic_label_fn("sort_by_label", qry_expr, labels)
}

/// MetricsQL's sort_by_label_desc query function
pub fn mql_sort_by_label_desc<'a>(
    qry_expr: impl Operable + 'a,
    labels: &'a [&'a str],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + 'a> {
    basic_label_fn("sort_by_label_desc", qry_expr, labels)
}

/// MetricsQL's sort_by_label_numeric query function
pub fn mql_sort_by_label_numeric<'a>(
    qry_expr: impl Operable + 'a,
    labels: &'a [&'a str],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + 'a> {
    basic_label_fn("sort_by_label_numeric", qry_expr, labels)
}
/// MetricsQL's sort_by_label_numeric_desc query function
pub fn mql_sort_by_label_numeric_desc<'a>(
    qry_expr: impl Operable + 'a,
    labels: &'a [&'a str],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + 'a> {
    basic_label_fn("sort_by_label_numeric_desc", qry_expr, labels)
}

#[cfg(test)]
mod tests {
    use crate::query::Metric;

    use super::*;

    #[test]
    fn label_label_map() {
        let query = mql_label_map(
            Metric::new("metric"),
            "label",
            &["val", "dest", "val1", "dest1"],
        )
        .to_string();

        assert_eq!(
            query,
            r#"label_map(metric,"label","val","dest","val1","dest1")"#
        );
    }

    #[test]
    fn label_sort_by_label() {
        let query = mql_sort_by_label(Metric::new("metric"), &["label", "label2"]).to_string();
        assert_eq!(query, r#"sort_by_label(metric,"label","label2")"#);
    }

    #[test]
    fn label_sort_by_label_desc() {
        let query = mql_sort_by_label_desc(Metric::new("metric"), &["label", "label2"]).to_string();
        assert_eq!(query, r#"sort_by_label_desc(metric,"label","label2")"#);
    }

    #[test]
    fn label_sort_by_label_numeric() {
        let query =
            mql_sort_by_label_numeric(Metric::new("metric"), &["label", "label2"]).to_string();
        assert_eq!(query, r#"sort_by_label_numeric(metric,"label","label2")"#);
    }

    #[test]
    fn label_sort_by_label_numeric_desc() {
        let query =
            mql_sort_by_label_numeric_desc(Metric::new("metric"), &["label", "label2"]).to_string();
        assert_eq!(
            query,
            r#"sort_by_label_numeric_desc(metric,"label","label2")"#
        );
    }
}
